﻿using BenchmarkDotNet.Attributes;
using BenchmarkDotNet.Running;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Bench
{
    public class Customer
    {
        public string Name { get; set; }
    }

    class Program
    {
        static void Main(string[] args)
        {
            BenchmarkRunner.Run<LinqBenchmarks>();
            //BenchmarkRunner.Run<StringEqualityBenchmark>();
        }
    }

    [MemoryDiagnoser]
    public class StringEqualityBenchmark
    {
        private string[] strings1 = new[] { "A", "B", "C", "D" };
        private string[] strings2 = new[] { "A", "E", "F", "D" };

        [Params(0, 1, 2, 3)]
        public int Index { get; set; }

        private string value1;
        private string value2;

        [GlobalSetup]
        public void GlobalSetup()
        {
            value1 = strings1[Index];
            value2 = strings2[Index];
        }

        [Benchmark]
        public bool EqualsOperator()
        {
            return value1 == value2;
        }

        [Benchmark]
        public bool EqualsMethod()
        {
            return value1.Equals(value2);
        }
    }

    [MemoryDiagnoser]
    [RPlotExporter]
    public class LinqBenchmarks
    {
        private HashSet<string> verifiedCustomerNames;
        private HashSet<string> certifiedCustomerNames;
        private HashSet<string> premiumCustomerNames;
        private Customer[] customersArray;
        private IEnumerable<Customer> customersEnumerable;

        [GlobalSetup]
        public void GlobalSetup()
        {
            verifiedCustomerNames = new() { "AAA", "BBB", "DDD" };
            certifiedCustomerNames = new() { "GGG", "HHH", "JJJ", "MMM", "NNN", "FFF" };
            premiumCustomerNames = new() { "CCC", "III", "KKK" };

            customersArray = new[]
            {
                new Customer() {Name = "Aaa"},
                new Customer() {Name = "Bbb"},
                new Customer() {Name = "Ccc"},
                new Customer() {Name = "Ooo"},
                new Customer() {Name = "Ppp"},
                new Customer() {Name = "Qqq"}
            };

            customersEnumerable = customersArray.ToList();
        }

        [Benchmark]
        public List<Customer> xrosecky()
        {
            var concat = verifiedCustomerNames.Concat(certifiedCustomerNames).Concat(premiumCustomerNames);
            return customersEnumerable.Where((c) => concat.Any((v) => c.Name.Equals(v))).ToList();
        }

        [Benchmark]
        public List<Customer> Kristian()
        {
            return customersEnumerable.Where(c => verifiedCustomerNames.Contains(c.Name.ToUpper())
                                                 || certifiedCustomerNames.Contains(c.Name.ToUpper())
                                                 || premiumCustomerNames.Contains(c.Name.ToUpper()))
                                      .ToList();
        }

        [Benchmark]
        public List<Customer> Riso()
        {
            var ver = verifiedCustomerNames.GroupBy(o => o)
                .Select(g => g.Key)
                .GroupBy(o => o)
                .Select(g => g.Key)
                .GroupBy(o => o)
                .Select(g => g.Key)
                .GroupBy(o => o)
                .Select(g => g.Key);
            return customersEnumerable.Where(customer => ver.Contains(customer.Name.ToUpper()) ||
                 certifiedCustomerNames.Contains(customer.Name.ToUpper()) ||
                 premiumCustomerNames.Contains(customer.Name.ToUpper()))
                .ToList();
        }

        [Benchmark]
        public List<Customer> ivo()
        {
            return customersArray
                .Where(c =>
                {
                    var upper = c.Name.ToUpper();
                    return verifiedCustomerNames.Contains(upper) || certifiedCustomerNames.Contains(upper) || premiumCustomerNames.Contains(upper);
                })
                .ToList();
        }

        [Benchmark(Baseline = true)]
        public List<Customer> For()
        {
            var result = new List<Customer>();

            for (int i = 0; i < customersArray.Length; i++)
            {
                var upper = customersArray[i].Name.ToUpper();

                if (verifiedCustomerNames.Contains(upper) || certifiedCustomerNames.Contains(upper) || premiumCustomerNames.Contains(upper))
                {
                    result.Add(customersArray[i]);
                }
            }

            return result;
        }
    }
}
